@@ -88,29 +88,31 @@ class TCPConnector:
             raise ConnectionPoolAcquireTimeout()
 
     async def after_acquire(self, urlparsed, conn, verify, ssl, timeouts, http2):
-        dns_info = await self.__resolve_dns(urlparsed.hostname, urlparsed.port)
 
         try:
+            dns_info = await self.__resolve_dns(urlparsed.hostname, urlparsed.port)
             await wait_for(
                 conn.connect(urlparsed, dns_info, verify, ssl, http2),
                 timeout=timeouts.sock_connect,
             )
         except TimeoutException:
+            self.release(conn)
             raise ConnectTimeout()
+        except BaseException as ex:
+            self.release(conn)
+            raise ex
         return conn
 
-    async def release(self, conn):
+    def release(self, conn):
         """Release connection."""
-        res = self.pool.release(conn)
-        if isinstance(res, Coroutine):
-            await res
+        self.pool.release(conn)
 
     async def wait_free_pool(self):
         """Wait until free pool."""
         while True:
             if self.pool.is_all_free():
                 return True
-            asyncio_sleep(0.02)  # pragma: no cover
+            await asyncio_sleep(0.02)  # pragma: no cover
 
     async def cleanup(self):
         """Cleanup connector connections."""
